/* system dependent stuff */

#ifdef OS2
/* using ANSI escape sequences wherever possible,
   but some Vio calls are necessary */
#define INCL_NOPM
#define INCL_SUB
#include <os2.h>
#include <conio.h>
#define KINFO KBDINFO
#endif

#ifdef LINUX
#include <termios.h>
#include <sys/ioctl.h>
#include <unistd.h>
#define KINFO struct termios
#endif

#ifdef WINDOWS
#include <windows.h>
#include <conio.h>
static HANDLE hscr = 0;
#endif

#ifdef CUBE
#include "ckernel.h"
#endif


#include "edt.h"

extern char headattr, tailattr, attrcache;
extern int xmax;
extern int h;

#define C_CHAR 0
#define C_ATTR 1
static char cell[3] = { ' ', 0, 0 }; /* char/attr/? (3rd byte for Windows) */

#ifndef WINDOWS
static KINFO kcook, kraw;
#endif

/* escape sequences (ANSI, VT100, cube) */
#define ESC_CLREOL     "\033[K"
#define ESC_SETATTR    "\033[%d;%dm"
#define ESC_SETCURS    "\033[%d;%dH"
#define ESC_SETDEFAULT "\033[0m"
#define ESC_SETHIGH    "\033[1m"
#define ESC_SETBLINK   "\033[5m"

#ifdef LINUX
#define ESC_SCROLLUP   "\033[%d;%dr\033[%d;1H\012"
#define ESC_SCROLLDN   "\033[%d;%dr\033[%d;1H\033M"
#endif
#ifdef CUBE
#define ESC_SCROLLUP   "\033[%d;%dS"
#define ESC_SCROLLDN   "\033[%d;%dT"
#endif


static void setattr( void )
{
#ifdef WINDOWS
   SetConsoleTextAttribute( hscr, cell[1] );
#else
static short ansicol[8] = { 0, 4, 2, 6, 1, 5, 3, 7 };
int fg, bg;

   if( cell[C_ATTR] != attrcache )
   {
      fg =  cell[C_ATTR] & 7;
      bg = (cell[C_ATTR] >> 4) & 7;

      printf( ESC_SETDEFAULT ESC_SETATTR, 30+ansicol[fg], 40+ansicol[bg] );
      if( cell[C_ATTR] & 8 )
         printf( ESC_SETHIGH );
      if( cell[C_ATTR] & 0x80 )
         printf( ESC_SETBLINK );
      attrcache = cell[C_ATTR];
   }
#endif
}


static void setcurs( int x, int y )
{
#ifdef WINDOWS
COORD pos;

   pos.X = x - 1;
   pos.Y = y - 1;
   SetConsoleCursorPosition( hscr, pos );
#else
   printf( ESC_SETCURS, y, x );
   fflush( stdout );
#endif
}


static void textattr( char na )
/* set character attribute */
{
   cell[C_ATTR] = na;
}


void stc( int x, int y)
/* set cursor position */
{
   setcurs( x, y );
   if( y <= 2 )
      textattr( headattr );
   else
      textattr( tailattr );
}


void ceol( void )
/* clear until end of line */
{
   setattr();
#ifdef WINDOWS
   {
   CONSOLE_SCREEN_BUFFER_INFO sb;
   int n, nact;

      GetConsoleScreenBufferInfo( hscr, &sb );
      n = sb.srWindow.Right - sb.dwCursorPosition.X + 1;
      if( n > 0 )
      {
         FillConsoleOutputCharacter( hscr, cell[C_CHAR], n, sb.dwCursorPosition, &nact );
         FillConsoleOutputAttribute( hscr, cell[C_ATTR], n, sb.dwCursorPosition, &nact );
      }
   }
#else
   printf( ESC_CLREOL );
   fflush( stdout );
#endif
}


void scrup( int a, int b)
/* scroll up */
{
#ifdef WINDOWS
SMALL_RECT rect;
COORD      dest;

   rect.Left  = 0;
   rect.Right = xmax-1;
   rect.Top   = a;
   rect.Bottom= b - 1;
   dest.X     = 0;
   dest.Y     = a - 1;
   ScrollConsoleScreenBuffer( hscr, &rect, 0, dest, (CHAR_INFO *)cell );
#elif OS2
   VioScrollUp( a-1, 0, b-1, xmax-1, 1, cell, 0 );
#else
   printf( ESC_SCROLLUP, a, b, b );
#endif
   stc(1,b);
}


void scrdn( int a, int b)
/* scroll down */
{
#ifdef WINDOWS
SMALL_RECT rect;
COORD      dest;

   rect.Left  = 0;
   rect.Right = xmax-1;
   rect.Top   = a - 1;
   rect.Bottom= b - 1;
   dest.X     = 0;
   dest.Y     = a;
   ScrollConsoleScreenBuffer( hscr, &rect, 0, dest, (CHAR_INFO *)cell );
#elif OS2
   VioScrollDn( a-1, 0, b-1, xmax-1, 1, cell, 0 );
#else
   printf( ESC_SCROLLDN, a, b, a );
#endif
   stc(1,a);
}


void stype( char * s )
/* type string */
{
   setattr();
   printf( "%s", s );
   fflush( stdout );
}


void getscrsize( int * w, int * h )
/* get screen size */
{
int rc;

#ifdef OS2
static VIOMODEINFO vm;

   vm.cb = sizeof(vm);
   rc = VioGetMode( &vm, 0 );
   if( rc == 0 )
   {
      *w = vm.col;
      *h = vm.row;
      return;
   }
#endif

#ifdef WINDOWS
CONSOLE_SCREEN_BUFFER_INFO sb;

   hscr = GetStdHandle( STD_OUTPUT_HANDLE );
   rc = GetConsoleScreenBufferInfo( hscr, &sb );
   if( rc )
   {
      *w = sb.srWindow.Right - sb.srWindow.Left + 1;
      *h = sb.srWindow.Bottom- sb.srWindow.Top  + 1;
      return;
   }
#endif

#ifdef LINUX
struct winsize size;

   rc = ioctl( 0, TIOCGWINSZ, &size );
   if( rc >= 0 )
   {
      *w = size.ws_col;
      *h = size.ws_row;
      return;
   }
#endif

/* default: */
   *w = 80;
   *h = 25;
}


int getchw( void )
{
register int c;

#if defined(OS2) || defined(WINDOWS)
   c = getch();
   if( c == 0xE0 )
      c = 0;
#endif

#ifdef LINUX
static char buf[8];
static int buflen = 0;
static int bufidx = 0;

   if( (bufidx >= 0) && (bufidx < buflen) )
   {
      c = buf[bufidx++];
   }
   else
   {
      buflen = read( 0, buf, sizeof(buf) );
/*
      printf( "%d chars: ", buflen );
      for( c = 0; c < buflen; c++ )
         printf( "%02X ", buf[c] );
*/
      if( (buflen > 1) && (buf[0] == 0x1B) )
      {
         bufidx = 2;
         c = 0;
         if( buflen == 4 )
         {
            if( buf[3] == '~' )
            {
               buflen = 3;
               if( buf[2] == '1' )
                  buf[2] = K2HOME;
               else if( buf[2] == '4' )
                  buf[2] = K2END;

            }
            else if( (buf[1] == '[') && (buf[2] == '[') )
            {
               if( buf[3] == 'A' )
               {
                  buflen = 3;
                  buf[2] = K2F1;
               }
            }
         }
      }
      else
      {
         bufidx = 1;
         c = buf[0];
      }
   }
   if( c == '\177' )
      c = '\b';

#endif

#ifdef CUBE
static long ch = 0;

   if( ch )
   {
      c = ch & 0xFF;
      ch = 0;
   }
   else
   {
      ch = ckTrmGetKey( 1 ) & KEY_CODE;
      if( ch > 0x100 )
         c = 0;
      else
      {
         c = ch;
         ch = 0;
      }
   }
#endif
   return c;
}


void kbraw( void )
{
#ifdef OS2
APIRET rc;

   kcook.cb = sizeof(kcook);
   rc = KbdGetStatus( &kcook, 0 );
   kraw = kcook;
   kraw.fsMask = (kraw.fsMask & 0xFFF0) | 0x6; /* binary mode, echo off */
   rc = KbdSetStatus( &kraw, 0 );
#endif

#ifdef LINUX
int rc;

   rc = tcgetattr( 0, &kcook );
   kraw = kcook;
   kraw.c_lflag &= ~(ECHO|ICANON|IEXTEN|ISIG);
   kraw.c_iflag &= ~(BRKINT|ICRNL);
   kraw.c_cc[VMIN]  = 1;
   kraw.c_cc[VTIME] = 0;
   rc = tcsetattr( 0, TCSAFLUSH, &kraw );
#endif
}


void kbcooked( void )
{
#ifdef OS2
APIRET rc;

   rc = KbdSetStatus( &kcook, 0 );
#endif

#ifdef LINUX
int rc;

   rc = tcsetattr( 0, TCSAFLUSH, &kcook );

   /* nun setzen wir das Scrollen wieder auf den ganzen Schirm -
      auch wenn das nichts mit der Tastatur zu tun hat: */
   printf( "\033[%d;%dr", 1, h );
   fflush( stdout );
#endif
}


